#ifndef BUILDING_PARAMETERS_H
#define BUILDING_PARAMETERS_H

#include "building.h"
#include "../share/common.h"
#include "../share/FileFactory.h"
#include "../simulator/PointSrc.h"

class BuildingParameters : public FileManager
{
    Building* ptr_blg;
    PointSrc* ptr_src;

    Vector2d e1, e2;		// Wind & cross wind direction unit vectors.
    double   ang;			// Angle made with effective building & the wind axis.
    Vector2d fr1, fr2, nrm;	// Front segment & internal normal of the effective building.
    double   www,lll;
    double   WW1,WW2,LLL;

	double h_h, w_1, w_2, h_e, d_x, d_y, z_1, z_2, y_1, y_2, a_l; int t_p, e_f;

	double max_gep;

	ofstream os;
protected:
	void setMaxGEP(double max_gep_){max_gep = max_gep_;}
	double getMaxGEP() const {return max_gep;}

	virtual double getWindDirectionALaMathematician(double blowing_direction_measured_from_north_with_clockwise_in_degrees) const;

    virtual double cor()    const {return SQRT_2*sin(getTheta() + M_PI*0.25);}		// Corrected factor for avoiding discontinuity.
    virtual double wb1()    const {return wb2()/cor();}
    virtual double wb2()    const;
    virtual double hbl()    const {return ptr_blg->getHb();}
    virtual double l_star() const {return min(wb1(), hbl());}

	virtual double distance(Tetragon ttr) const;
	virtual void   setTheta(double ang_) {ang = ang_;}
	virtual double getTheta(bool in_deg = false) const {return (in_deg ? RAD2DEG*ang : ang);}
	virtual void   front();
	virtual void   setFront(Vector2d  fr1_, Vector2d  fr2_, Vector2d  nrm_)       {fr1  = fr1_; fr2  = fr2_; nrm  = nrm_;}
	virtual void   getFront(Vector2d& fr1_, Vector2d& fr2_, Vector2d& nrm_) const {fr1_ = fr1 ; fr2_ = fr2 ; nrm_ = nrm ;}
	virtual void   setwl(double  www_, double  lll_)       {www = www_; lll = lll_;}
	virtual void   getwl(double& www_, double& lll_) const {www_ = www; lll_ = lll;}
	virtual void   setWL(double  WW1_, double  WW2_, double  LLL_)       {WW1 = WW1_; WW2 = WW2_; LLL = LLL_;}
	virtual void   getWL(double& WW1_, double& WW2_, double& LLL_) const {WW1_ = WW1; WW2_ = WW2; LLL_ = LLL;}

    virtual double gepStackHeight() const {return hbl() + 1.5*min(hbl(), wb2());}
    virtual bool   isTallStack() const;
    virtual void   lookupEffectiveBuilding();

    virtual void   evaluate();
	virtual double hef() const;
	virtual double alp() const;
	virtual int    typ() const;
	virtual int    eff() const {return e_f;}
	virtual void   getCs(double& z_1_, double& z_2_, double& y_1_, double& y_2_) const;

	virtual void   getVs(double& d_x_, double& d_y_) const;			// Highly experimental implementation!!!
	virtual double crt() const {return 0.25*wb1();}					// Critical distance whether evaluating virtual source position or not.
	virtual double scl() const {return 0.5*cos(getTheta())*wb1();}	// Correction scale
	virtual double smt() const
	{
		const double deg = getTheta(true);
		return (deg > 20 ? 1.0 : 1 - cos((deg*4.5)*DEG2RAD));
	}	// Smoother for avoiding discontinuity.

    virtual void   dump(int w_d);
public:
    BuildingParameters(Building* ptr_blg_)
    {
		setIsFile(FileList::fnBuildingParameters(FileList::IS, 1));
		setOsFile(FileList::fnBuildingParameters(FileList::OS, 1));
		ptr_blg = ptr_blg_;
	}
    virtual void setPointSrc(PointSrc* ptr_src_){ptr_src = ptr_src_;}
    virtual void exec ();
    virtual void open (){const string filename = getOsFile(); os.open(filename.c_str());}
    virtual void close(){os.close();}
    virtual ~BuildingParameters(){}
};

#endif
