#include "../share/common.h"
#include "ErrorHandler.h"
#include "MetData.h"

MetData::~MetData()
{
	if(getNoElms() > 0)
	{
		head();
		delete [] yr;
		delete [] mn;
		delete [] dy;
		delete [] hr;

		delete [] wd;
		delete [] ws;
		delete [] tp;

		delete [] dl;
		delete [] sn;

		setNoElms(0); setPosition(0); nr_days = 0;
	}
}

void MetData::setMetData(int  yr_, int  mn_, int  dy_, int  hr_, double  wd_, double  ws_, double  tp_, double  dl_, double  sn_)
{
	*yr = yr_;
	*mn = mn_;
	*dy = dy_;
	*hr = hr_;

	*wd = wd_;
	*ws = ws_;
	*tp = tp_;

	*dl = dl_;
	*sn = sn_;
}

void MetData::getMetData(int& yr_, int& mn_, int& dy_, int& hr_, double& wd_, double& ws_, double& tp_, double& dl_, double& sn_) const
{
	yr_ = *yr;
	mn_ = *mn;
	dy_ = *dy;
	hr_ = *hr;

	wd_ = *wd;
	ws_ = *ws;
	tp_ = *tp;

	dl_ = *dl;
	sn_ = *sn;
}

void MetData::redim(int n)
{
	if(getNoElms() > 0)
	{
		head();
		delete [] yr;
		delete [] mn;
		delete [] dy;
		delete [] hr;
		delete [] wd;
		delete [] ws;
		delete [] tp;
		delete [] dl;
		delete [] sn;
		setNoElms(0); setPosition(0); nr_days = 0;
	}

	yr = new int[n+1];
	mn = new int[n+1];
	dy = new int[n+1];
	hr = new int[n+1];
	wd = new double[n+1];
	ws = new double[n+1];
	tp = new double[n+1];
	dl = new double[n+1];
	sn = new double[n+1];
	setNoElms(n); setPosition(1); nr_days = 0;
}

void MetData::init()
{
	head();
	for(int i = 1; i<=getNoElms(); i++)
	{
		*yr = 0;
		*mn = 0;
		*dy = 0;
		*hr = 0;
		*wd = 0;
		*ws = 0;
		*tp = 0;
		*dl = 0;
		*sn = 0;
		advance();
	}
	head();
}

void MetData::advance()
{
	SimpleIterator::advance();
	yr++;
	mn++;
	dy++;
	hr++;
	wd++;
	ws++;
	tp++;
	dl++;
	sn++;
}

void MetData::reverse()
{
	SimpleIterator::reverse();
	yr--;
	mn--;
	dy--;
	hr--;
	wd--;
	ws--;
	tp--;
	dl--;
	sn--;
}

void MetData::scan()
{
	const string filename = getIsFile(); ifstream is(filename.c_str());

	is.exceptions(ios_base::goodbit);
	try{
		is.exceptions(ios_base::badbit | ios_base::failbit);

		is >> lon;
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is >> lat;
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is >> zzz;
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');

		int n; is >> n; redim(n);
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');

		head(); nr_days = 0; int prev_dy = 0;
		while(getPosition() <= getNoElms())
		{
			int _yr;
			int _mn;
			int _dy;
			int _hr;

			double _wd;
			double _ws;
			double _tp;

			double _dl;
			double _sn;

			is >> _yr;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _mn;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _dy;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _hr;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _wd;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _ws;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _tp;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _dl;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _sn;

			*yr = _yr;
			*mn = _mn;
			*dy = _dy;
			*hr = _hr;
			*wd = _wd;
			*ws = _ws;
			*tp = _tp;
			*dl = _dl; if(_dl > 1.001 && _dl < 10.001) throw ErrorMetData_scan_dl(filename, _dl);
			*sn = _sn;

			if(_dy != prev_dy)
			{
				nr_days++;
				prev_dy = _dy;
			}

			advance();
		}
		head();
		is.close();
	}
	catch(const ErrorHandler& e)
	{
		cerr << e;
		CERR << e;
		exit(1);
	}
	catch(...)
	{
		cerr << ErrorMetData_scan(filename);
		CERR << ErrorMetData_scan(filename);
		exit(1);
	}
}

void MetData::print()
{
#if SCAN_CHECK > 0
	const string filename = getOsFile();

	ofstream os(filename.c_str());

	os << setw(COLUMN) << lon << ',' << setw(COLUMN) << lat << ',' << setw(COLUMN) << zzz << ",,,,,," << endl;

	os << "<< Scanned meteorological data (yr,mn,dy,hr,wd,ws,tp,dl,sn). " << getNoElms() << " data readed. >>" << ",," << endl;
	os << endl;

	head();
	while(getPosition() <= getNoElms())
	{
		os << setw(COLUMN) << *yr << ',' << setw(COLUMN) << *mn << ',' << setw(COLUMN) << *dy << ',' << setw(COLUMN) << *hr << ',' << setw(COLUMN) << *wd << ',' << setw(COLUMN) << *ws << ',' << setw(COLUMN) << *tp << ',' << setw(COLUMN) << *dl << ',' << setw(COLUMN) << *sn << endl;
		advance();
	}
	head();
	os.close();
#endif
}

void MetData::lists()
{
	CLST << endl;
	CLST << endl;
	CLST << "<< Scanned meteorological data list >>" << endl;
	CLST << endl;

	CLST << "+ OBS POSITION +" << endl;
	CLST << "^^^^^^^^^^^^^^^^" << endl;
	CLST << setw(COLUMN) << "Longitude (deg)" << setw(COLUMN) << "Latitude (deg)" << setw(COLUMN) << "Anemometer Ht. (m)" << endl;
	CLST << "------------------------------------------------------------" << endl;
	CLST << setw(COLUMN) << lon << setw(COLUMN) << lat << setw(COLUMN) << zzz << endl;
	CLST << endl;

	CLST << "+ RAW METEOROLOGICAL DATA +" << endl;
	CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
	CLST << setw(COLUMN) << "Year" << setw(COLUMN) << "Month" << setw(COLUMN) << "Day" << setw(COLUMN) << "Hour" << setw(COLUMN) << "Wind dir (0-16)" << setw(COLUMN) << "Wind speed (m/s)" << setw(COLUMN) << "Temperature (K)" << setw(COLUMN) << "Sunshine (0-1)" << setw(COLUMN) << "SRad (0.01MJ/m^2/h)" << endl;
	CLST << "------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------" << endl;

	head();
	while(getPosition() <= getNoElms())
	{
		CLST << setw(COLUMN) << *yr << setw(COLUMN) << *mn << setw(COLUMN) << *dy << setw(COLUMN) << *hr << setw(COLUMN) << *wd << setw(COLUMN) << *ws << setw(COLUMN) << *tp << setw(COLUMN) << *dl << setw(COLUMN) << *sn << endl;
		advance();
	}
	CLST << endl;
	head();
}
