#include "../share/common.h"
#include "ErrorHandler.h"
#include "MetData2.h"


void MetData2::scan()
{
	const string filename = getIsFile(); ifstream is(filename.c_str());

	is.exceptions(ios_base::goodbit);
	try{
		is.exceptions(ios_base::badbit | ios_base::failbit);

		is >> lon;
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is >> lat;
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is >> zzz;
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		if(JP)
			tzn = 9;
		else
			is >> tzn;
		setTimeZone(tzn);
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');

		int n; is >> n; redim(n);
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');

		head(); nr_days = 0; int prev_dy = 0;
		while(getPosition() <= getNoElms())
		{
			int _yr;
			int _mn;
			int _dy;
			int _hr;

			double _wd;
			double _ws;
			double _tp;

			double _dl;
			double _sn;

			is >> _yr;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _mn;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _dy;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _hr;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _wd;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _ws;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _tp;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _dl;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _sn;

			*yr = _yr;
			*mn = _mn;
			*dy = _dy;
			*hr = _hr;
			*wd = _wd;
			*ws = _ws;
			*tp = _tp;
			*dl = _dl; if(_dl > 1.001 && _dl < 10.001) throw ErrorMetData_scan_dl(filename, _dl);
			*sn = _sn;

			if(_dy != prev_dy)
			{
				nr_days++;
				prev_dy = _dy;
			}

			advance();
		}
		head();
		is.close();
	}
	catch(const ErrorHandler& e)
	{
		cerr << e;
		CERR << e;
		exit(1);
	}
	catch(...)
	{
		cerr << ErrorMetData_scan(filename);
		CERR << ErrorMetData_scan(filename);
		exit(1);
	}
}

void MetData2::print()
{
#if SCAN_CHECK > 0
	const string filename = getOsFile();

	ofstream os(filename.c_str());

	os << setw(COLUMN) << lon << ',' << setw(COLUMN) << lat << ',' << setw(COLUMN) << zzz << ',' << setw(COLUMN) << tzn << ",,,,," << endl;

	os << "<< Scanned meteorological data (yr,mn,dy,hr,wd,ws,tp,dl,sn). " << getNoElms() << " data readed. >>" << ",," << endl;
	os << endl;

	head();
	while(getPosition() <= getNoElms())
	{
		os << setw(COLUMN) << *yr << ',' << setw(COLUMN) << *mn << ',' << setw(COLUMN) << *dy << ',' << setw(COLUMN) << *hr << ',' << setw(COLUMN) << *wd << ',' << setw(COLUMN) << *ws << ',' << setw(COLUMN) << *tp << ',' << setw(COLUMN) << *dl << ',' << setw(COLUMN) << *sn << endl;
		advance();
	}
	head();
	os.close();
#endif
}

void MetData2::lists()
{
	CLST << endl;
	CLST << endl;
	CLST << "<< Scanned meteorological data list >>" << endl;
	CLST << endl;

	CLST << "+ OBS POSITION +" << endl;
	CLST << "^^^^^^^^^^^^^^^^" << endl;
	CLST << setw(COLUMN) << "Longitude (deg)" << setw(COLUMN) << "Latitude (deg)" << setw(COLUMN) << "Anemometer Ht. (m)" << setw(COLUMN) << "Time Zone" << endl;
	CLST << "--------------------------------------------------------------------------------" << endl;
	CLST << setw(COLUMN) << lon << setw(COLUMN) << lat << setw(COLUMN) << zzz << setw(COLUMN) << getTimeZone() << endl;
	CLST << endl;

	CLST << "+ RAW METEOROLOGICAL DATA +" << endl;
	CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
	CLST << setw(10) << "Year" << setw(10) << "Month" << setw(10) << "Day" << setw(10) << "Hour" << setw(10) << "Wind dir" << setw(10) << "Wind spd" << setw(10) << "Temp." << setw(10) << "Sunshine" << setw(20) << "Short Rad"      << endl;
	CLST << setw(10) << ""     << setw(10) << ""      << setw(10) << ""    << setw(10) << ""     << setw(10) << "(0-16)"   << setw(10) << "(m/s)"    << setw(10) << "(K)"   << setw(10) << "(0-1)"    << setw(20) << "(0.01MJ/m^2/h)" << endl;
	CLST << "----------------------------------------------------------------------------------------------------" << endl;

	head();
	while(getPosition() <= getNoElms())
	{
		CLST << setw(10) << *yr << setw(10) << *mn << setw(10) << *dy << setw(10) << *hr << setw(10) << *wd << setw(10) << *ws << setw(10) << *tp << setw(10) << *dl << setw(20) << *sn << endl;
		advance();
	}
	CLST << endl;
	head();
}
