#include "stability.h"
#include "statistics.h"


Stability::~Stability()
{

	delete tim;
	delete sun;
	delete met;
}

void Stability::scan()
{
	met = new MetData();
	met->scan();
	met->getGeoPosition(lon, lat, zzz);

	sun = new Sun(lon, lat);

	tim = new TimeCode();
	tim->scan();

	met->lists();
	tim->lists();
}

void Stability::print() const
{
	met->print();
}

void Stability::run()
{
	// Create average daylight table
	const int nr = met->getNoDays();
	double* average = new double [nr];

	met->head(); int seq = 0; int prev_dy = 0; int cnt = 0; double sum = 0.0; int i;
	for(i = 1; i <= met->getNoElms(); i++)
	{
		// Placeholders for getting the meteorological data
		int _yr, _mn, _dy, _hr; double _wd, _ws, _tp, _dl, _sn;
		met->getMetData(_yr, _mn, _dy, _hr, _wd, _ws, _tp, _dl, _sn);

		if(i == 1)
		{
			prev_dy = _dy;
		}
		else if(_dy != prev_dy)
		{
			average[seq++] = (cnt != 0 ? sum/cnt : 9999); sum = cnt = 0; prev_dy = _dy;
			// cout << seq-1 << ',' << average[seq-1] << endl;
		}

		if((_hr >= 8 && _hr <= 17) && (_dl >= 0.0 && _dl <= 1.0))
		{
			sum += _dl;
			cnt++;
		}
		met->advance();
	}
	average[seq++] = (cnt != 0 ? sum/cnt : 9999); met->head();

	const string filename = getOsFile(); ofstream os(filename.c_str());


	CLST << endl;
	CLST << endl;
	CLST << "<< Evaluated stability parameters list >>" << endl;
	CLST << endl;
	CLST << "+ MODEL-READY METEOROLOGICAL DATA +" << endl;
	CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
	CLST << setw(COLUMN) << "Day class" << setw(COLUMN) << "Hour class" << setw(COLUMN) << "Year" << setw(COLUMN) << "Month" << setw(COLUMN) << "Day" << setw(COLUMN) << "Hour" << setw(COLUMN) << "Wind dir (0-16)" << setw(COLUMN) << "Wind speed (m/s)" << setw(COLUMN) << "PG-params(1-11)" << setw(COLUMN) << "Temperature (K)" << setw(COLUMN) << "SRad (0.01MW/m^2/h)" << endl;
	CLST << "----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------" << endl;

	Statistics* stat = new Statistics(tim->getMaxDailyCodes(), tim->getMaxHourlyCodes());

	met->head(); seq = 0; prev_dy = 0;
	for(i = 1; i <= met->getNoElms(); i++)
	{
		// Placeholders for getting the meteorological data
		int _yr, _mn, _dy, _hr; double _wd, _ws, _tp, _dl, _sn;
		met->getMetData(_yr, _mn, _dy, _hr, _wd, _ws, _tp, _dl, _sn);

		if(i == 1)
		{
			prev_dy = _dy;
		}
		else if(_dy != prev_dy)
		{
			seq++; prev_dy = _dy;
		}
		sun->setDate(_yr, _mn, _dy); sun->setTime(_hr); const Sun::SunPosition sp = sun->getSunPosition();


		// Evaluate stalilities (First step)
		int res = table(sp, _dl, _ws, average[seq], _sn);

		// Evaluate stalilities (Second step. Some modification done with corrected wind speed)
		if(res != 9999)
		{
			double pp;
			if(res <= 2)
				pp = 0.07;
			else if(res <= 4)
				pp = 0.07;
			else if(res <= 6)
				pp = 0.10;
			else if(res <= 8)
				pp = 0.15;
			else if(res == 9)
				pp = 0.35;
			else
				pp = 0.55;

			const double ws = _ws*pow(10.0/zzz, pp);
			res = table(sp, _dl, ws, average[seq], _sn);
		}

		tim->setTime(_yr, _mn, _dy, _hr);
		const int _Dy = tim-> getDailyCode();
		const int _Hr = tim->getHourlyCode();

		os << setw(COLUMN) << _Dy << ',';
		os << setw(COLUMN) << _Hr << ',';
		os << setw(COLUMN) << _yr << ',';
		os << setw(COLUMN) << _mn << ',';
		os << setw(COLUMN) << _dy << ',';
		os << setw(COLUMN) << _hr << ',';
		os << setw(COLUMN) << _wd << ',';
		os << setw(COLUMN) << _ws << ',';
		os << setw(COLUMN) << res << ',';
		os << setw(COLUMN) << _tp << ',';
		os << setw(COLUMN) << _sn;
		os << endl;

		// Append to list file
		CLST << setw(COLUMN) << _Dy;
		CLST << setw(COLUMN) << _Hr;
		CLST << setw(COLUMN) << _yr;
		CLST << setw(COLUMN) << _mn;
		CLST << setw(COLUMN) << _dy;
		CLST << setw(COLUMN) << _hr;
		CLST << setw(COLUMN) << _wd;
		CLST << setw(COLUMN) << _ws;
		CLST << setw(COLUMN) << res;
		CLST << setw(COLUMN) << _tp;
		CLST << setw(COLUMN) << _sn;
		CLST << endl;

		stat->add(_Dy, _Hr, _wd, _ws, res);

		met->advance();
	}
	met->head(); os.close(); stat->dump(); delete stat; CLST << endl; delete [] average;
}

int Stability::table(Sun::SunPosition sp, double dl, double ws, double ave, double sn) const
{
	const double coef = 3.6;	// Conversion coefficient : 0.01kW/m^2 -> 0.01MJ/m^2/h
	int res; double rad; /* int trace = 0; */

	switch(sp)
	{
	case Sun::ABOVE:
		if(sn < 0.0)
		{
			if(dl > 0 && dl <= 1.0)
			{
				rad = sun->radiation(dl); /* trace = 1; */
			}
			else if(dl == 0)
			{
				if(ave != 9999)
					{rad = sun->radiation(-ave); /* trace = 2; */}
				else
					{rad = 9999.0; /* cout << "pass1" << endl; */}
			}
			else
			{
				rad = 9999.0; /* cout << "pass2" << endl; */
			}
		}
		else
		{
			rad = (sn >= 9000 ? 9999.0 : sn);
		}

		/* cout << "trace=" << trace << " dl=" << dl << " ave=" << ave << " rad=" << rad << endl; */

		if(rad >= 60*coef && rad < 500*coef)
		{
			if(ws < 2.0)
				res = 1;
			else if(ws < 3.0)
				res = 2;
			else if(ws < 4.0)
				res = 3;
			else if(ws < 9000)
				res = 5;
			else
				res = 9999;
			break;
		}
		if(rad >= 30*coef && rad < 60*coef)
		{
			if(ws < 2.0)
				res = 2;
			else if(ws < 3.0)
				res = 3;
			else if(ws < 4.0)
				res = 4;
			else if(ws < 6.0)
				res = 6;
			else if(ws < 9000)
				res = 7;
			else
				res = 9999;
			break;
		}
		if(rad >= 15*coef && rad < 30*coef)
		{
			if(ws < 2.0)
				res = 3;
			else if(ws < 4.0)
				res = 5;
			else if(ws < 9000)
				res = 7;
			else
				res = 9999;
			break;
		}
		if(rad > 0 && rad < 15*coef)
		{
			res = 7;
			break;
		}
		res = 9999;
		break;
	case Sun::HORIZON_DD:
		res = 7;
		break;
	case Sun::BELOW:
		if(ws < 2.0)
			res = 10;
		else if(ws < 3.0)
			res = 9;
		else if(ws < 4.0)
			res = 8;
		else if(ws < 9000)
			res = 8;
		else
			res = 9999;
		break;
	case Sun::HORIZON_DN:
		res = 8;
		break;
	}
	return res;
}
