#include "statistics.h"

Statistics::Statistics(int nr_dy_codes_, int nr_hr_codes_)
{
	nr_dy_codes = nr_dy_codes_; nr_dy_codes_p = nr_dy_codes + 1;
	nr_hr_codes = nr_hr_codes_; nr_hr_codes_p = nr_hr_codes + 1;
	nr_wd_codes = 17;
	nr_st_codes = 11;

	const int array_size_1 = nr_dy_codes_p*nr_hr_codes_p*nr_wd_codes;
	const int array_size_2 = nr_dy_codes_p*nr_hr_codes_p*nr_st_codes;
	const int array_size_3 = nr_dy_codes_p*nr_hr_codes_p;

	windspeed             = new double [array_size_1];
	accepted_wind_counter = new int    [array_size_1];
	rejected_wind_counter = new int    [array_size_3];
	accepted_stab_counter = new int    [array_size_2];
	rejected_stab_counter = new int    [array_size_3];

	for(int i = 0; i < array_size_1; i++)
	{
		accepted_wind_counter[i] = 0; windspeed[i] = 0.0;
	}
	for(int j = 0; j < array_size_2; j++)
	{
		accepted_stab_counter[j] = 0;
	}
	for(int k = 0; k < array_size_3; k++)
	{
		rejected_wind_counter[k] = rejected_stab_counter[k] = 0;
	}
}

Statistics::~Statistics()
{
	delete [] rejected_stab_counter;
	delete [] accepted_stab_counter;
	delete [] rejected_wind_counter;
	delete [] accepted_wind_counter;
	delete [] windspeed;
}

void Statistics::reset()
{
	const int array_size_1 = nr_dy_codes_p*nr_hr_codes_p*nr_wd_codes;
	const int array_size_2 = nr_dy_codes_p*nr_hr_codes_p*nr_st_codes;
	const int array_size_3 = nr_dy_codes_p*nr_hr_codes_p;

	for(int i = 0; i < array_size_1; i++)
	{
		accepted_wind_counter[i] = 0; windspeed[i] = 0.0;
	}
	for(int j = 0; j < array_size_2; j++)
	{
		accepted_stab_counter[j] = 0;
	}
	for(int k = 0; k < array_size_3; k++)
	{
		rejected_wind_counter[k] = rejected_stab_counter[k] = 0;
	}
}

int  Statistics::getWindDirectionCode(double wd) const
{
	if(false)         {/* dummy */}
	else if(fabs(wd) <  1.0e-6) {return 17;}
	else if(wd       >      17) {return 9999;}
	else                        {return floor(wd + 0.5);}
}

string Statistics::wdsymbol(int wd) const
{
	switch(wd)
	{
	case  1:
		return "NNE";
	case  2:
		return "NE";
	case  3:
		return "ENE";
	case  4:
		return "E";
	case  5:
		return "ESE";
	case  6:
		return "SE";
	case  7:
		return "SSE";
	case  8:
		return "S";
	case  9:
		return "SSW";
	case 10:
		return "SW";
	case 11:
		return "WSW";
	case 12:
		return "W";
	case 13:
		return "WNW";
	case 14:
		return "NW";
	case 15:
		return "NNW";
	case 16:
		return "N";
	case 17:
		return "CALM";
	default:
		return "N/A";
	}
}

string Statistics::stsymbol(int st) const
{
	switch(st)
	{
	case  1:
		return "A";
	case  2:
		return "AB";
	case  3:
		return "B";
	case  4:
		return "BC";
	case  5:
		return "C";
	case  6:
		return "CD";
	case  7:
		return "DD";
	case  8:
		return "DN";
	case  9:
		return "E";
	case 10:
		return "F";
	case 11:
		return "G";
	default:
		return "N/A";
	}
}

void Statistics::add(int dy, int hr, double wd, double ws, int st)
{
	const int idx_dy_hr = (hr            - 1) + (dy            - 1)*nr_hr_codes_p;
	const int idx_dy    = (nr_hr_codes_p - 1) + (dy            - 1)*nr_hr_codes_p;
	const int idx_hr    = (hr            - 1) + (nr_dy_codes_p - 1)*nr_hr_codes_p;
	const int idx       = (nr_hr_codes_p - 1) + (nr_dy_codes_p - 1)*nr_hr_codes_p;

	const int WD = getWindDirectionCode(wd);

	const int idx_wd_dy_hr = WD - 1 +  (hr            - 1)*nr_wd_codes + (dy            - 1)*nr_hr_codes_p*nr_wd_codes;
	const int idx_wd_dy    = WD - 1 +  (nr_hr_codes_p - 1)*nr_wd_codes + (dy            - 1)*nr_hr_codes_p*nr_wd_codes;
	const int idx_wd_hr    = WD - 1 +  (hr            - 1)*nr_wd_codes + (nr_dy_codes_p - 1)*nr_hr_codes_p*nr_wd_codes;
	const int idx_wd       = WD - 1 +  (nr_hr_codes_p - 1)*nr_wd_codes + (nr_dy_codes_p - 1)*nr_hr_codes_p*nr_wd_codes;

	if(WD != 9999 && ws < 999.0)
	{
		accepted_wind_counter[idx_wd_dy_hr]++;
		accepted_wind_counter[idx_wd_dy   ]++;
		accepted_wind_counter[idx_wd_hr   ]++;
		accepted_wind_counter[idx_wd      ]++;
		windspeed            [idx_wd_dy_hr] += ws;
		windspeed            [idx_wd_dy   ] += ws;
		windspeed            [idx_wd_hr   ] += ws;
		windspeed            [idx_wd      ] += ws;
	}
	else
	{
		rejected_wind_counter[idx_dy_hr]++;
		rejected_wind_counter[idx_dy   ]++;
		rejected_wind_counter[idx_hr   ]++;
		rejected_wind_counter[idx      ]++;
	}

	const int idx_st_dy_hr = st - 1 +  (hr            - 1)*nr_st_codes + (dy            - 1)*nr_hr_codes_p*nr_st_codes;
	const int idx_st_dy    = st - 1 +  (nr_hr_codes_p - 1)*nr_st_codes + (dy            - 1)*nr_hr_codes_p*nr_st_codes;
	const int idx_st_hr    = st - 1 +  (hr            - 1)*nr_st_codes + (nr_dy_codes_p - 1)*nr_hr_codes_p*nr_st_codes;
	const int idx_st       = st - 1 +  (nr_hr_codes_p - 1)*nr_st_codes + (nr_dy_codes_p - 1)*nr_hr_codes_p*nr_st_codes;

	if(st > 0 && st < 12)
	{
		accepted_stab_counter[idx_st_dy_hr]++;
		accepted_stab_counter[idx_st_dy   ]++;
		accepted_stab_counter[idx_st_hr   ]++;
		accepted_stab_counter[idx_st      ]++;
	}
	else
	{
		rejected_stab_counter[idx_dy_hr]++;
		rejected_stab_counter[idx_dy   ]++;
		rejected_stab_counter[idx_hr   ]++;
		rejected_stab_counter[idx      ]++;
	}
}

void Statistics::dump() const
{
	int dy, hr, wd, st; int nr_of_samples_wrt_wind, nr_of_samples_wrt_stab;

	for(dy = 1; dy <= nr_dy_codes_p; dy++)
	{
		for(hr = 1; hr <= nr_hr_codes_p; hr++)
		{
			const int idx_dy_hr = (hr - 1) + (dy - 1)*nr_hr_codes_p;

			nr_of_samples_wrt_wind = 0;
			for(wd = 1; wd <= nr_wd_codes; wd++)
			{
				const int idx_wd_dy_hr  = wd - 1 + (hr - 1)*nr_wd_codes + (dy - 1)*nr_hr_codes_p*nr_wd_codes;
				nr_of_samples_wrt_wind += accepted_wind_counter[idx_wd_dy_hr];
			}

			nr_of_samples_wrt_stab = 0;
			for(st = 1; st <= nr_st_codes; st++)
			{
				const int idx_st_dy_hr  = st - 1 + (hr - 1)*nr_st_codes + (dy - 1)*nr_hr_codes_p*nr_st_codes;
				nr_of_samples_wrt_stab += accepted_stab_counter[idx_st_dy_hr];
			}

			const int hours = nr_of_samples_wrt_wind + rejected_wind_counter[idx_dy_hr];

			CLST << endl;
			CLST << endl;
			CLST << "                    *          *         *                    " << endl;
			CLST << endl;
			CLST << endl;
			CLST << setw(40) << "Season Class                       : " << setw(10)  << dy                     << endl;
			CLST << setw(40) << "Time   Class                       : " << setw(10)  << hr                     << endl;
			CLST << setw(40) << "Total  Hours                       : " << setw(10)  << hours                  << endl;
			CLST << setw(40) << "Number of Available WD and WS Data : " << setw(10)  << nr_of_samples_wrt_wind << endl;
			CLST << setw(40) << "Number of Available Stability Data : " << setw(10)  << nr_of_samples_wrt_stab << endl;
			CLST << endl;


			CLST << setw(20) << "WIND DIRECTION" << setw(20) << "FREQUENCY (%)" << setw(20) << "AVERAGE (m/s)" << endl;
			CLST << "------------------------------------------------------------" << endl;
			for(wd = 1; wd <= nr_wd_codes; wd++)
			{
				const int    idx_wd_dy_hr = wd - 1 + (hr - 1)*nr_wd_codes + (dy - 1)*nr_hr_codes_p*nr_wd_codes;
				const double w_ave = windspeed[idx_wd_dy_hr]/accepted_wind_counter[idx_wd_dy_hr];
				const double w_frq = (double)accepted_wind_counter[idx_wd_dy_hr]/(double)nr_of_samples_wrt_wind;
				CLST << setw(20) << wdsymbol(wd) << setw(20) << 0.01*floor(10000*w_frq + 0.5) << setw(20) << 0.1*floor(10*w_ave + 0.5) << endl;
			}

			const double na_wd = (double)rejected_wind_counter[idx_dy_hr]/hours;
			CLST << "------------------------------------------------------------" << endl;
			CLST << setw(20) << "N/A" << setw(20) << 0.01*floor(10000*na_wd + 0.5) << setw(20) << "---" << endl;
			CLST << endl;


			CLST << endl;
			CLST << setw(20) << "PG-SYMBOL"      << setw(20) << "FREQUENCY (%)" <<  endl;
			CLST << "----------------------------------------" << endl;
			for(st = 1; st <= nr_st_codes; st++)
			{
				const int    idx_st_dy_hr = st - 1 + (hr - 1)*nr_st_codes + (dy - 1)*nr_hr_codes_p*nr_st_codes;
				const double s_frq = (double)accepted_stab_counter[idx_st_dy_hr]/(double)nr_of_samples_wrt_stab;
				CLST << setw(20) << stsymbol(st) << setw(20) << 0.01*floor(10000*s_frq + 0.5) << endl;
			}

			const double na_st = (double)rejected_stab_counter[idx_dy_hr]/hours;
			CLST << "----------------------------------------" << endl;
			CLST << setw(20) << "N/A" << setw(20) << 0.01*floor(10000*na_st + 0.5) << endl;
			CLST << endl;
		}
	}
}
