#include "sun2.h"


double Sun2::hourAngle() const
{
	const int hr = getTime(); const double minutes = hr*60 - 30;
	double sr, ss; evalSunRiseSunSet(sr, ss);
	const double culm = (sr + ss)*0.5;
	return (minutes - culm)*0.25*M_PI/180;
}

Sun::SunPosition Sun2::getSunPosition(double& sr, double& ss) const
{
	const int hr = getTime(); const int minutes = hr*60;

	evalSunRiseSunSet(sr, ss);

	if(minutes < (sr + 60))
	{
		if(minutes < sr)
			return BELOW;
		else
			return HORIZON_DN;
	}
	if(minutes > ss - 60)
	{
		if(minutes > ss)
			return BELOW;
		else
			return HORIZON_DN;
	}

	if(
		minutes > sr + 120 &&
		minutes < ss - 120)
		return ABOVE;
	else
		return HORIZON_DD;
}

void Sun2::evalSunRiseSunSet(double& sr, double& ss) const
{
	int  YY, MM, DD; getDate(YY, MM, DD);
	double lat, lon; getLatLon(lat, lon); lon = - lon; /* East is negative, west is positive! */

	const double lhour = lon/15.0;
	const double phi   = lat;

	const int y  = (MM > 2 ? YY : YY -  1);
	const int m  = (MM > 2 ? MM : MM + 12);
	const int jd = floor(365.25*y) + floor(30.6001*(m  + 1)) + DD + 1720994.5;
	const int a  = floor(y/100);
	const int b  = 2 - a + floor(a*0.25);
	const double jdt = jd + b;

	const int m0 = 13;
	const int j0 = floor(365.25*y) + floor(30.6001*(m0 + 1))      + 1720994.5;
	const int a0 = floor(y/100);
	const int b0 = 2 - a0 + floor(a0*0.25);
	const double jd0 = j0 + b0;

	const int N = jdt - jd0;

	const double tm = N + ( 6 + lhour)/24.0;
	const double te = N + (18 + lhour)/24.0;

	const double HH = 0.0;	// HH : Altitude. For simplicity, HH fixed to the sea level;

	const double z0 = 90 + (50 + 2.12*sqrt(HH))/60.0;





	double M, lambda, alpha, delta, x0, H0, T0;

	M = 0.985600*tm - 3.289;
	lambda = M + 1.916*sin(M*DEG2RAD) + 0.020*sin(2.0*M*DEG2RAD) + 282.634;
	lambda = modulo(lambda, 360);
	alpha  = RAD2DEG*atan2(0.91746*sin(lambda*DEG2RAD), cos(lambda*DEG2RAD))/15.0;
	delta  = RAD2DEG*asin (0.39782*sin(lambda*DEG2RAD));
	x0     = (cos(z0*DEG2RAD) - sin(delta*DEG2RAD)*sin(phi*DEG2RAD))/(cos(delta*DEG2RAD)*cos(phi*DEG2RAD));
	H0     = (360 - RAD2DEG*acos(x0))/15.0;
	T0     = H0 + alpha - 0.065710*tm - 6.622;
	T0     = modulo(T0, 24);
	T0     = (T0 < 0 ? T0 + 24 : T0);
	sr     = (T0 + lhour + getTimeZone())*60;

	M = 0.985600*te - 3.289;
	lambda = M + 1.916*sin(M*DEG2RAD) + 0.020*sin(2.0*M*DEG2RAD) + 282.634;
	lambda = modulo(lambda, 360);
	alpha  = RAD2DEG*atan2(0.91746*sin(lambda*DEG2RAD), cos(lambda*DEG2RAD))/15.0;
	delta  = RAD2DEG*asin (0.39782*sin(lambda*DEG2RAD));
	x0     = (cos(z0*DEG2RAD) - sin(delta*DEG2RAD)*sin(phi*DEG2RAD))/(cos(delta*DEG2RAD)*cos(phi*DEG2RAD));
	H0     = RAD2DEG*acos(x0)/15.0;
	T0     = H0 + alpha - 0.065710*te - 6.622;
	T0     = modulo(T0, 24);
	T0     = (T0 < 0 ? T0 + 24 : T0);
	ss     = (T0 + lhour + getTimeZone())*60;

	//cout << sr/60 << ',' << ss/60 << endl;
}
