#ifndef ERROR_HANDLER_H
#define ERROR_HANDLER_H

#include "../share/common.h"

// Prototype declaration(s)
class ErrorHandler;
class Simulator;


// Define global function(s) for binary operators
ostream& operator<<(ostream&  os, const ErrorHandler& e);
ostream& operator<<(ofstream& os, const ErrorHandler& e);


// Definition(s) of interface class(es)
class ErrorHandler
{
public:
	virtual string message() const = 0;
	virtual ~ErrorHandler(){}
};




// Definition(s) of concrete class(es)

class ErrorMetData_scan : public ErrorHandler
{
	string str;
public:
	ErrorMetData_scan(string str_){str = str_;}
	virtual string message() const;
};

class ErrorTimeCode_scan : public ErrorHandler
{
	string str;
public:
	ErrorTimeCode_scan(string str_){str = str_;}
	virtual string message() const;
};

class ErrorBlg_scan : public ErrorHandler
{
	string str;
public:
	ErrorBlg_scan(string str_){str = str_;}
	virtual string message() const;
};

class ErrorDroplet_scan : public ErrorHandler
{
	string str;
public:
	ErrorDroplet_scan(string str_){str = str_;}
	virtual string message() const;
};

class ErrorLineSrc_scan : public ErrorHandler
{
	string str;
public:
	ErrorLineSrc_scan(string str_){str = str_;}
	virtual string message() const;
};


class ErrorPasquillGifford_szSolve : public ErrorHandler
{
	double sol;
	double rhs;
public:
	ErrorPasquillGifford_szSolve(double sol_, double rhs_){sol = sol_; rhs = rhs_;}
	virtual string message() const;
};

class ErrorPasquillGifford_getPrm_z : public ErrorHandler
{
	int st;
public:
	ErrorPasquillGifford_getPrm_z(int st_){st = st_;}
	virtual string message() const;
};

class ErrorPasquillGifford_getPrm_y : public ErrorHandler
{
	int st;
public:
	ErrorPasquillGifford_getPrm_y(int st_){st = st_;}
	virtual string message() const;
};

class ErrorPasquillGifford_getPrm_sySolve : public ErrorHandler
{
	int st;
public:
	ErrorPasquillGifford_getPrm_sySolve(int st_){st = st_;}
	virtual string message() const;
};

class ErrorPasquillGifford_getTurners : public ErrorHandler
{
	int st;
public:
	ErrorPasquillGifford_getTurners(int st_){st = st_;}
	virtual string message() const;
};

class ErrorMain_argc : public ErrorHandler
{
	int nr;
public:
	ErrorMain_argc(int nr_){nr = nr_;}
	virtual string message() const;
};

class ErrorMain_argfile : public ErrorHandler
{
	string str;
public:
	ErrorMain_argfile(string str_){str = str_;}
	virtual string message() const;
};

class ErrorMain_argv : public ErrorHandler
{
	string op1;
	string op2;
public:
	ErrorMain_argv(string op1_, string op2_){op1 = op1_; op2 = op2_;}
	virtual string message() const;
};

class ErrorMet_scan : public ErrorHandler
{
	string str;
public:
	ErrorMet_scan(string str_){str = str_;}
	virtual string message() const;
};

class ErrorMetTimeSeries_scan : public ErrorHandler
{
	string str;
public:
	ErrorMetTimeSeries_scan(string str_){str = str_;}
	virtual string message() const;
};

class ErrorNR_gser_1 : public ErrorHandler
{
	double x;
public:
	ErrorNR_gser_1(double x_){x = x_;}
	virtual string message() const;
};

class ErrorNR_gser_2 : public ErrorHandler
{
	double a;
	int    i;
public:
	ErrorNR_gser_2(double a_, int i_){a = a_; i = i_;}
	virtual string message() const;
};

class ErrorNR_gcf : public ErrorHandler
{
	double a;
	int    i;
public:
	ErrorNR_gcf(double a_, int i_){a = a_; i = i_;}
	virtual string message() const;
};

class ErrorNR_gammp : public ErrorHandler
{
	double a;
	double x;
public:
	ErrorNR_gammp(double a_, double x_){a = a_; x = x_;}
	virtual string message() const;
};

class ErrorNR_gammq : public ErrorHandler
{
	double a;
	double x;
public:
	ErrorNR_gammq(double a_, double x_){a = a_; x = x_;}
	virtual string message() const;
};

class ErrorNR_chebev : public ErrorHandler
{
	double a;
	double b;
	double x;
public:
	ErrorNR_chebev(double a_, double b_, double x_){a = a_; b = b_; x = x_;}
	virtual string message() const;
};

class ErrorNR_bessik_1 : public ErrorHandler
{
	double n;
	double x;
public:
	ErrorNR_bessik_1(double n_, double x_){n = n_; x = x_;}
	virtual string message() const;
};

class ErrorNR_bessik_2 : public ErrorHandler
{
	double x;
public:
	ErrorNR_bessik_2(double x_){x = x_;}
	virtual string message() const;
};

class ErrorNR_bessik_3 : public ErrorHandler
{
	int i;
public:
	ErrorNR_bessik_3(int i_){i = i_;}
	virtual string message() const;
};

class ErrorNR_bessik_4 : public ErrorHandler
{
	int i;
public:
	ErrorNR_bessik_4(int i_){i = i_;}
	virtual string message() const;
};

class ErrorPat_open : public ErrorHandler
{
	string str;
public:
	ErrorPat_open(string str_){str = str_;}
	virtual string message() const;
};

class ErrorPat_scan : public ErrorHandler
{
	string str;
public:
	ErrorPat_scan(string str_){str = str_;}
	virtual string message() const;
};

class ErrorPointDropletSrc_scan : public ErrorHandler
{
	string str;
public:
	ErrorPointDropletSrc_scan(string str_){str = str_;}
	virtual string message() const;
};

class ErrorPointSrc_scan : public ErrorHandler
{
	string str;
public:
	ErrorPointSrc_scan(string str_){str = str_;}
	virtual string message() const;
};

class ErrorRan_open : public ErrorHandler
{
	string str;
public:
	ErrorRan_open(string str_){str = str_;}
	virtual string message() const;
};

class ErrorRan_scan : public ErrorHandler
{
	string str;
public:
	ErrorRan_scan(string str_){str = str_;}
	virtual string message() const;
};

class ErrorRcp_scan : public ErrorHandler
{
	string str;
public:
	ErrorRcp_scan(string str_){str = str_;}
	virtual string message() const;
};

class ErrorSimulationFactory_scan : public ErrorHandler
{
	string str;
public:
	ErrorSimulationFactory_scan(string str_){str = str_;}
	virtual string message() const;
};

class ErrorSimulationFactory_loop : public ErrorHandler
{
	int  total;
	int  delim;
	int  steps;
	bool incrm;
public:
	ErrorSimulationFactory_loop(int total_, int delim_, int steps_, bool incrm_ = false)
	{
		total = total_;
		delim = delim_;
		steps = steps_;
		incrm = incrm_;
	}
	virtual string message() const;
};

class ErrorSimulator_scan : public ErrorHandler
{
	string str;
public:
	ErrorSimulator_scan(string str_){str = str_;}
	virtual string message() const;
};

class ErrorSimulator_power4ws : public ErrorHandler
{
	int i;
public:
	ErrorSimulator_power4ws(int i_){i = i_;}
	virtual string message() const;
};

class ErrorSimpson : public ErrorHandler
{
	double     tol;
	Simulator* dat;
	bool       icr;
public:
	ErrorSimpson(double tol_, Simulator* dat_, bool icr_ = false){tol = tol_; dat = dat_; icr = icr_;}
	virtual string message() const;
};

class ErrorTerrainAdjustment_simpl : public ErrorHandler
{
	int    i_d;
	double x_x;
	double y_y;
	double e_x;
public:
	ErrorTerrainAdjustment_simpl(int i_d_, double x_x_, double y_y_, double e_x_){i_d = i_d_; x_x = x_x_; y_y = y_y_; e_x = e_x_;}
	virtual string message() const;
};

#endif
