#include "MetTimeSeries.h"
#include "ErrorHandler.h"
#include "../share/common.h"

MetTimeSeries::~MetTimeSeries()
{
	ML_TRACE("MetTimeSeries::~MetTimeSeries");

	if(getNoElms() > 0)
	{
		head();
		delete [] yr;
		delete [] mn;
		delete [] dy;
		delete [] hr;
	}
}

void MetTimeSeries::setTime(int  yr_, int  mn_, int  dy_, int  hr_)
{
	ML_TRACE("MetTimeSeries::setTime");

	*yr = yr_;
	*mn = mn_;
	*dy = dy_;
	*hr = hr_;
}

void MetTimeSeries::getTime(int& yr_, int& mn_, int& dy_, int& hr_) const
{
	ML_TRACE("MetTimeSeries::getTime");

	yr_ = *yr;
	mn_ = *mn;
	dy_ = *dy;
	hr_ = *hr;
}

void MetTimeSeries::redim(int n)
{
	ML_TRACE("MetTimeSeries::redim");

	if(getNoElms() > 0)
	{
		head();
		delete [] yr;
		delete [] mn;
		delete [] dy;
		delete [] hr;
		Met::~Met();
	}

	Met::redim(n);
	yr = new int[n+1];
	mn = new int[n+1];
	dy = new int[n+1];
	hr = new int[n+1];
}

void MetTimeSeries::advance()
{
	Met::advance();
	yr++;
	mn++;
	dy++;
	hr++;
}

void MetTimeSeries::reverse()
{
	Met::reverse();
	yr--;
	mn--;
	dy--;
	hr--;
}

void MetTimeSeries::scan()
{
	ML_TRACE("MetTimeSeries::scan");

	const string filename = getIsFile();
	ifstream  is(filename.c_str());

	is.exceptions(ios_base::goodbit);
	try{
		is.exceptions(ios_base::badbit | ios_base::failbit);

		double meters; is >> meters; setAnemometerHeight(meters);
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		int n; is >> n; redim(n);
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');

		head();
		for(int i = 1; i <= getNoElms(); i++)
		{
			int _yr;
			int _mn;
			int _dy;
			int _hr;

			double _wd;
			double _ws;
			int    _st;
			double _tp;

			is >> _yr;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _mn;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _dy;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _hr;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _wd;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _ws;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _st;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _tp;

			*yr = _yr;
			*mn = _mn;
			*dy = _dy;
			*hr = _hr;
			*wd = wdConverter(_wd);
			*ws = _ws;
			*st = _st;
			*tp = _tp;
			advance();
		}
		head();
		is.close();
	}
	catch(...)
	{
		cerr << ErrorMetTimeSeries_scan(filename);
		CERR << ErrorMetTimeSeries_scan(filename);
		exit(1);
	}
}

void MetTimeSeries::print()
{
#if SCAN_CHECK > 0
	const string filename = getOsFile();
	ofstream  os(filename.c_str());

	os << "<< Scanned meteorological data (yr|mn|dy|hr|wd|ws|st|tp). " << getNoElms() << " data readed. >>" << "," << getAnemometerHeight() << ",,,,,," << endl;
	os << endl;

	head();
	for(int i = 1; i <=getNoElms(); i++)
	{
		os << setw(COLUMN) << *yr << ',' << setw(COLUMN) << *mn << ',' << setw(COLUMN) << *dy << ',' << setw(COLUMN) << *hr << ',' << setw(COLUMN) << wdConverter(*wd) << ',' << setw(COLUMN) << *ws << ',' << setw(COLUMN) << *st << ',' << setw(COLUMN) << *tp << endl;
		advance();
	}
	head();
	os.close();
#endif
}

void MetTimeSeries::lists()
{
	ML_TRACE("MetTimeSeries::lists");

	CLST << endl;
	CLST << endl;
	CLST << "<< Meteorological data list >>" << endl;
	CLST << endl;
	CLST << "* Stability Code Table *" << endl;
	CLST << "^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
	CLST << setw(10) << "Code" << setw(10) << "PG Symb" << setw(10) << "Used as" << endl;
	CLST << "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~" << endl;
	CLST << setw(10) << " 1" << setw(10) << " A" << setw(10) << " A" << endl;
	CLST << setw(10) << " 2" << setw(10) << "AB" << setw(10) << " A" << endl;
	CLST << setw(10) << " 3" << setw(10) << " B" << setw(10) << " B" << endl;
	CLST << setw(10) << " 4" << setw(10) << "BC" << setw(10) << " B" << endl;
	CLST << setw(10) << " 5" << setw(10) << " C" << setw(10) << " C" << endl;
	CLST << setw(10) << " 6" << setw(10) << "CD" << setw(10) << " C" << endl;
	CLST << setw(10) << " 7" << setw(10) << "DD" << setw(10) << "DD" << endl;
	CLST << setw(10) << " 8" << setw(10) << "DN" << setw(10) << "DN" << endl;
	CLST << setw(10) << " 9" << setw(10) << " E" << setw(10) << " E" << endl;
	CLST << setw(10) << "10" << setw(10) << " F" << setw(10) << " F" << endl;
	CLST << setw(10) << "11" << setw(10) << " G" << setw(10) << " F" << endl;
	CLST << "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~" << endl;
	CLST << endl;
	CLST << "+ METEOROLOGICAL DATA (LONG TERM) +" << endl;
	CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
	CLST << setw(COLUMN) << "Year" << setw(COLUMN) << "Month" << setw(COLUMN) << "Day" << setw(COLUMN) << "Hour" << setw(COLUMN) << "Wind dir (deg)" << setw(COLUMN) << "Wind Speed (m/s)" << setw(COLUMN) << "Stability (1-11)" << setw(COLUMN) << "Temperature (K)" << endl;
	CLST << "----------------------------------------------------------------------------------------------------------------------------------------------------------------" << endl;
	head();
	for(int i = 1; i <=getNoElms(); i++)
	{
		CLST << setw(COLUMN) << *yr << setw(COLUMN) << *mn << setw(COLUMN) << *dy << setw(COLUMN) << *hr << setw(COLUMN) << wdConverter(*wd) << setw(COLUMN) << *ws << setw(COLUMN) << *st << setw(COLUMN) << *tp << endl;
		advance();
	}
	head(); CLST << endl;
}

