#include "MetiLisPasquillGifford.h"


void MetiLisPasquillGifford::setParameters(double w1_,double w2_,double hh_,double z1_,double z2_,double y1_,double y2_)
{
	ML_TRACE("MetiLisPasquillGifford::setParameters");

	_w1_ = w1_; _w2_ = w2_; _hh_ = hh_;
	_z1_ = z1_; _z2_ = z2_; _y1_ = y1_; _y2_ = y2_;

	PG_Q = (_hh_ < 1.0e-3);


	if(PG_Q)
	{
		ML_TRACE("Using PG dispersion width");
		sz_sol = sy_sol = 0.0;
	}
	else
	{
		ML_TRACE("Using METI-LIS dispersion width");

		ML_TRACE(_z1_); ML_TRACE(_z2_);
		ML_TRACE(_y1_); ML_TRACE(_y2_);

		// Evaluate vertical virtual distance.
		const double SZ1 = el()*(_z1_+ 7.0*_z2_);
		const double pgz = PasquillGifford::z(10.0*el());
		const double sz1 = max(pgz, SZ1);
		//const double sz1 = el()*(_z1_+ 7.0*_z2_);

		szSolve(sz_sol, sz1);
		if(TRACE_Q)
		{
			CLOG << "(sz_sol, rhs_z)=" << sz_sol << ',' << el()*(_z1_+ 7.0*_z2_) << endl;
		}


		// Evaluate horizontal virtual distance.
		if(false){/* dummy */}
		else if(_w2_ > 5.0*_hh_)
		{
			y_offset = _hh_;
			cy1_coef = _hh_;
			cy2_coef = _hh_;
		}
		else if(_w2_ < _hh_)
		{
			y_offset = _w2_;
			cy1_coef = _w2_;
			cy2_coef = _w2_;
		}
		else
		{
			y_offset = _hh_;
			cy1_coef = _w2_;
			cy2_coef = _hh_;
		}

		const double SY1 = cy1_coef*_y1_ + 7.0*cy2_coef*_y2_;
		const double pgy = PasquillGifford::y(10.0*y_offset);
		const double sy1 = max(pgy, SY1);
		//const double sy1 = cy1_coef*_y1_ + 7.0*cy2_coef*_y2_;

		sySolve(sy_sol, sy1/getDilutionFactor());
		if(TRACE_Q)
		{
			cout << "sy1=" << sy1 << " pgy=" << PasquillGifford::y(10.0*y_offset) << endl;
			CLOG << "(sy_sol, rhs_y)=" << sy_sol << ',' << sy1/getDilutionFactor() << endl;
		}
		if(VALIDATION_Q)
		{
			CLST << endl;
			CLST << endl;
			CLST << setw(30) << "params=";
			CLST << setw(10) << _w1_;
			CLST << setw(10) << _w2_;
			CLST << setw(10) << _hh_;
			CLST << setw(10) << _z1_;
			CLST << setw(10) << _z2_;
			CLST << setw(10) << _y1_;
			CLST << setw(10) << _y2_;
			CLST << endl;

			CLST << setw(30) << "10*el,SZ,pgz,sz,sol,add=";
			CLST << setw(10) << 10*el();
			CLST << setw(10) << SZ1;
			CLST << setw(10) << pgz;
			CLST << setw(10) << sz1;
			CLST << setw(20) << sz_sol;
			CLST << setw(20) << max(0.0, sz_sol - 10*el());
			CLST << endl;
			CLST << setw(30) << "10*of,SY,pgy,sy,sy',sol,add=";
			CLST << setw(10) << 10*y_offset;
			CLST << setw(10) << SY1;
			CLST << setw(10) << pgy;
			CLST << setw(10) << sy1;
			CLST << setw(10) << sy1/getDilutionFactor();
			CLST << setw(10) << sy_sol;
			CLST << setw(20) << max(0.0, sy_sol - 10*y_offset);
			CLST << endl;
			CLST << endl;

		}
	}
}

double MetiLisPasquillGifford::z(double x)
{
	ML_TRACE("MetiLisPasquillGifford::z");

	if(PG_Q)
	{
		if(VALIDATION_Q)
		{
			CLST << "formula_z pg" << endl;
		}

		return PasquillGifford::z(x);
	}
	else
	{
		if(x < 10.0*el())
		{
			if(x < 3.0*el()) not_validated_wrt_z = true;
			const double pgz = PasquillGifford::z(x);
			const double sz1 = _z1_*el() + _z2_*(x - 3.0*el());

			if(VALIDATION_Q)
			{
				if(pgz >= sz1)
					CLST << "formula_z pg" << endl;
				else
					CLST << "formula_z mt" << endl;
			}

			return max(pgz, sz1);
		}
		else
		{
			if(VALIDATION_Q)
			{
				CLST << "formula_z pg" << endl;
			}

			return PasquillGifford::z(x + max(0.0, sz_sol - 10.0*el()));
		}
	}
}

double MetiLisPasquillGifford::y(double x)
{
	ML_TRACE("MetiLisPasquillGifford::y");

	if(PG_Q)
	{
		if(VALIDATION_Q)
		{
			CLST << endl;
			CLST << endl;
			CLST << "formula_y pg" << endl;
		}

		return PasquillGifford::y(x);
	}
	else
	{
		const double epsilon = 1.0e-3;

		if(x < 10.0*y_offset + epsilon)
		{
			double sy1;

			if(false){/* dummy */}
			else if(_w2_ > 5.0*_hh_)
			{
				sy1 = _y1_*_hh_ + _y2_*(x - 3.0*_hh_);
			}
			else if(_w2_ < _hh_)
			{
				sy1 = _y1_*_w2_ + _y2_*(x - 3.0*_w2_);
			}
			else
			{
				sy1 = _y1_*_w2_ + _y2_*(x - 3.0*_hh_);
			}

			if(x < 3.0*y_offset + epsilon) not_validated_wrt_y = true;

			const double pgy = PasquillGifford::y(x);

			if(VALIDATION_Q)
			{
				CLST << endl;
				CLST << endl;
				if(pgy >= sy1)
					CLST << "formula_y pg" << endl;
				else
					CLST << "formula_y mt" << endl;
			}

			return max(pgy, sy1);
		}
		else
		{
			if(VALIDATION_Q)
			{
				CLST << endl;
				CLST << endl;
				CLST << "formula_y pg" << endl;
			}

			return PasquillGifford::y(x + max(0.0, sy_sol - 10.0*y_offset));
		}
	}
}
