#include "PasquillGifford.h"
#include "ErrorHandler.h"
#include "../share/common.h"

double PasquillGifford::y(double x)
{
	ML_TRACE("PasquillGifford::y");

	const double km = 0.001*x;
	double c, d; getPrm_y(c, d);
	const double th = 0.017453293*(c - d*log(km));
	return 465.11628*km*tan(th)*getDilutionFactor();
}

double PasquillGifford::z(double x)
{
	ML_TRACE("PasquillGifford::z");

	const double km = 0.001*x;
	double a, b; getPrm_z(a, b, km);
	const double sz = a*pow(km, b);
	return (sz > 5000.0 && st <= 3 ? 5000.0 : sz);
}

void PasquillGifford::sySolve(double& sol, double rhs)
{
	ML_TRACE("PasquillGifford::sySolve");

	if(rhs == 0){sol = 0; return;}

	double p, q; getPrm_sySolve(p, q);
	sol = pow(rhs/p, 1.0/q)*1000;
}

void PasquillGifford::szSolve(double& sol, double rhs)
{
	ML_TRACE("PasquillGifford::szSolve");

	if(rhs == 0){sol = 0; return;}

	double a,b;
	double predicted;
	double floor_dist;

	predicted = floor_dist = 0.0;
	while(1)							// Bugfixed. 2005/10/19
	{
		getPrm_z(a, b, floor_dist);
		predicted = pow(rhs/a, 1.0/b);
		if(predicted >= floor_dist && predicted < ceil_dist)
		{
			sol = predicted*1000;
			// CLOG << st << ',' << floor_dist << ',' << predicted << ',' << ceil_dist << ',' << rhs - PasquillGifford::z(sol) << '\n';
			return;
		}

		if(floor_dist >= ceil_dist)
		{
			CERR << "Failed in evaluating the virtical virtual distance. Process aborted.\n";
			exit(1);
		}
		else
			floor_dist = ceil_dist;
	}

}

void PasquillGifford::getPrm_z(double& a, double& b, double km)
{
	ML_TRACE("PasquillGifford::getPrm_z");

	switch(st)
	{
	case 1:
		if(km < setCeilDist(0.10))
		{
			a = 122.800; b = 0.94470; return;
		}
		else if(km < setCeilDist(0.15))
		{
			a = 158.080; b = 1.05420; return;
		}
		else if(km < setCeilDist(0.20))
		{
			a = 170.220; b = 1.09320; return;
		}
		else if(km < setCeilDist(0.25))
		{
			a = 179.520; b = 1.12620; return;
		}
		else if(km < setCeilDist(0.30))
		{
			a = 217.410; b = 1.26440; return;
		}
		else if(km < setCeilDist(0.40))
		{
			a = 258.890; b = 1.40940; return;
		}
		else if(km < setCeilDist(0.50))
		{
			a = 346.750; b = 1.72830; return;
		}
		else if(km < setCeilDist(3.11))
		{
			a = 453.850; b = 2.11660; return;
		}
		else
		{
			a = 999.999; b = 3.00000; ceil_dist = 9999.0; return;
		}
	case 2:
		if(km < setCeilDist(0.20))
		{
			a =  90.673; b = 0.93198; return;
		}
		else if(km < setCeilDist(0.40))
		{
			a =  98.483; b = 0.98332; return;
		}
		else
		{
			a = 109.300; b = 1.09710; ceil_dist = 9999.0; return;
		}
	case 3:
		    a =  61.141; b = 0.91465; ceil_dist = 9999.0; return;
	case 4:
		if(km < setCeilDist(0.30))
		{
			a =  34.459; b = 0.86974; return;
		}
		else if(km < setCeilDist(1.00))
		{
			a =  32.093; b = 0.81066; return;
		}
		else if(km < setCeilDist(3.00))
		{
			a =  32.093; b = 0.64403; return;
		}
		else if(km < setCeilDist(10.00))
		{
			a =  33.504; b = 0.60486; return;
		}
		else if(km < setCeilDist(30.00))
		{
			a =  36.650; b = 0.56589; return;
		}
		else
		{
			a =  44.053; b = 0.51179; ceil_dist = 9999.0; return;
		}
	case 5:
		if(km < setCeilDist(0.10))
		{
			a =  24.260; b = 0.83660; return;
		}
		else if(km < setCeilDist(0.30))
		{
			a =  23.331; b = 0.81956; return;
		}
		else if(km < setCeilDist(1.00))
		{
			a =  21.628; b = 0.75660; return;
		}
		else if(km < setCeilDist(2.00))
		{
			a =  21.628; b = 0.63077; return;
		}
		else if(km < setCeilDist(4.00))
		{
			a =  22.534; b = 0.57154; return;
		}
		else if(km < setCeilDist(10.00))
		{
			a =  24.703; b = 0.50527; return;
		}
		else if(km < setCeilDist(20.00))
		{
			a =  26.970; b = 0.46713; return;
		}
		else if(km < setCeilDist(40.00))
		{
			a =  35.420; b = 0.37615; return;
		}
		else
		{
			a =  47.618; b = 0.29592; ceil_dist = 9999.0; return;
		}
	case 6:
		if(km < setCeilDist(0.20))
		{
			a =  15.209; b = 0.81558; return;
		}
		else if(km < setCeilDist(0.70))
		{
			a =  14.457; b = 0.78407; return;
		}
		else if(km < setCeilDist(1.00))
		{
			a =  13.953; b = 0.68465; return;
		}
		else if(km < setCeilDist(2.00))
		{
			a =  13.953; b = 0.63227; return;
		}
		else if(km < setCeilDist(3.00))
		{
			a =  14.823; b = 0.54503; return;
		}
		else if(km < setCeilDist(7.00))
		{
			a =  16.187; b = 0.46490; return;
		}
		else if(km < setCeilDist(15.00))
		{
			a =  17.836; b = 0.41507; return;
		}
		else if(km < setCeilDist(30.00))
		{
			a =  22.651; b = 0.32681; return;
		}
		else if(km < setCeilDist(60.00))
		{
			a =  27.074; b = 0.27436; return;
		}
		else
		{
			a =  34.219; b = 0.21716; ceil_dist = 9999.0; return;
		}
	default:
		cerr << ErrorPasquillGifford_getPrm_z(st);
		CERR << ErrorPasquillGifford_getPrm_z(st);
		exit(1);
	}
}

void PasquillGifford::getPrm_y(double& c, double& d)
{
	ML_TRACE("PasquillGifford::getPrm_y");

	switch(st)
	{
	case 1:
		c = 24.1670; d = 2.5334 ; return;
	case 2:
		c = 18.3330; d = 1.8096 ; return;
	case 3:
		c = 12.5000; d = 1.0857 ; return;
	case 4:
		c =  8.3330; d = 0.72382; return;
	case 5:
		c =  6.2500; d = 0.54287; return;
	case 6:
		c =  4.1667; d = 0.36191; return;
	default:
		cerr << ErrorPasquillGifford_getPrm_y(st);
		CERR << ErrorPasquillGifford_getPrm_y(st);
		exit(1);
	}
}

void PasquillGifford::getPrm_sySolve(double& p, double& q)
{
	ML_TRACE("PasquillGifford::getPrm_sySolve");

	switch(st)
	{
	case 1:
		p = 209.14; q = 0.890; return;
	case 2:
		p = 154.46; q = 0.902; return;
	case 3:
		p = 103.26; q = 0.917; return;
	case 4:
		p =  68.26; q = 0.919; return;
	case 5:
		p =  51.06; q = 0.921; return;
	case 6:
		p =  33.92; q = 0.919; return;
	default:
		cerr << ErrorPasquillGifford_getPrm_sySolve(st);
		CERR << ErrorPasquillGifford_getPrm_sySolve(st);
		exit(1);
	}
}

void PasquillGifford::getTurners(double& alp, double& gam)
{
	ML_TRACE("PasquillGifford::getTurners");

	switch(st)
	{
	case 1:
		alp = 0.948; gam = 1.569; return;
	case 2:
		alp = 0.781; gam = 0.474; return;
	case 3:
		alp = 0.635; gam = 0.208; return;
	case 4:
		alp = 0.470; gam = 0.113; return;
	case 5:
		alp = 0.439; gam = 0.067; return;
	case 6:
		alp = 0.439; gam = 0.048; return;
	default:
		cerr << ErrorPasquillGifford_getTurners(st);
		CERR << ErrorPasquillGifford_getTurners(st);
		exit(1);
	}
}
