#ifndef POINT_DROPLET_SRC_H
#define POINT_DROPLET_SRC_H

#include "PointSrc.h"
#include "droplet.h"

#define MAX_DROPLETS 100

class WeightRatio
{
	double ratio[MAX_DROPLETS];
	int    nr_of_elms;
public:
	WeightRatio(){nr_of_elms = MAX_DROPLETS;}
	virtual int    getNoElms() const {return nr_of_elms;}
	virtual void   setWeightRatio(int n, double ratio_){n <= getNoElms() ? ratio[n-1] = ratio_ : throw;}
	virtual double getWeightRatio(int n) const         {return ratio[n-1];}
	virtual ~WeightRatio(){}
};

class BulkDensity
{
	double density[MAX_DROPLETS];
	int    nr_of_elms;
public:
	BulkDensity(){nr_of_elms = MAX_DROPLETS;}
	virtual int    getNoElms() const {return nr_of_elms;}
	virtual void   setBulkDensity(int n, double density_){n <= getNoElms() ? density[n-1] = density_ : throw;}
	virtual double getBulkDensity(int n) const           {return density[n-1];}
	virtual ~BulkDensity(){}
};

class PointDropletSrc : public PointSrc
{
	Droplet*     droplet;
	BulkDensity* bulk_density;
	WeightRatio* weight_ratio;
protected:
	double* drg;
public:
	PointDropletSrc(Droplet* droplet_)
	{
		droplet = droplet_;
		setIsFile(FileList::fnPointDropletSrc(FileList::IS, 1));
		setOsFile(FileList::fnPointDropletSrc(FileList::OS, 1));
	}
	virtual void redim(int n);
	virtual void scan ();
	virtual void print();
	virtual void lists();
	virtual void advance();
	virtual void reverse();

	virtual void   getMediumProperties(double& drg_) const {drg_ = *drg;}
	virtual double getBulkDensity(int i) const {return bulk_density->getBulkDensity(i);}
	virtual double getWeightRatio(int i) const {return weight_ratio->getWeightRatio(i);}
	virtual ~PointDropletSrc();
};

#endif
