#ifndef POINT_DROPLET_SRC_SIMULATO_H
#define POINT_DROPLET_SRC_SIMULATOR_H

#include "PointSrcSimulator.h"
#include "PointDropletSrc.h"

class PointDropletSrcSimulator : public PointSrcSimulator
{
protected:
	enum PollutantProperty {GAS, DROPLET};
	virtual void   setPollutantProperty(PollutantProperty prp_){prp = prp_;}
	virtual PollutantProperty getPollutantProperty() const     {return prp;}
private:
	double droplet_radius;
	double dia;
	double dns;
	double drg;

	double con;
	double dep;

	double atz;
	double at0;

	PollutantProperty prp;
protected:
	Droplet* droplet;
protected:
	virtual double getGravitationalSettlingVelocity() const;
	virtual double getDepositionVelocity() const;
	virtual double dh() const;

	virtual void   setDropletDensity (double dns_){dns = dns_;}
	virtual double getDropletDensity () const {return dns;}
	virtual void   setDropletDrag    (double drg_){drg = drg_;}
	virtual double getDropletDrag    () const {return drg;}
	virtual void   setDropletDiameter(double dia_in_meter){dia = dia_in_meter; droplet_radius = 0.5*dia_in_meter;}
	virtual double getDropletDiameter() const {return dia;}
	virtual double getDropletRadius  () const {return droplet_radius;}

	virtual double alpha(double h_e);
	virtual double equation(PlumePuff pp = PLUME);

	virtual void   evalAndAddThisDropletContrib(double wgt);

	virtual void   setReceptorConcentrationAtZ(double atz_) {atz = atz_;}
	virtual double getReceptorConcentrationAtZ() const      {return atz;}
	virtual void   setReceptorConcentrationAt0(double at0_) {at0 = at0_;}
	virtual double getReceptorConcentrationAt0() const      {return at0;}
public:
	PointDropletSrcSimulator(){droplet = new Droplet();}

	virtual ~PointDropletSrcSimulator(){delete droplet;}
	virtual void scan ();
	virtual void print();
	virtual void lists();
	virtual void exec (bool new_met, bool new_src);
};


#endif