#ifndef POINT_SRC_SIMULATOR_H
#define POINT_SRC_SIMULATOR_H

#include "simulator.h"
#include "PointSrc.h"

class PointSrcSimulator : public Simulator
{
	bool     stk;
	bool     plm;
	double   emi;
	double   vel;
	double   gas;
	double   tpr;
	double   dia;
	Vector3d ctr;

	int      ide;
	double   alt;

	double   dH;

	double  downwind_distance;
	double crosswind_distance;
protected:
	virtual double equation(PlumePuff pp = PLUME);
	virtual int    assembleSimulationProperties(bool new_met, bool new_src);

	virtual void   setDownwindDistance (double downwind_distance_){downwind_distance = downwind_distance_;}
	virtual double getDownwindDistance () const {return max(1.0e-6, downwind_distance);}	// For preventing zero devision!
	virtual void   setCrosswindDistance(double crosswind_distance_){crosswind_distance = crosswind_distance_;}
	virtual double getCrosswindDistance() const {return crosswind_distance;}

	virtual void     setStk(bool   stk_){stk = stk_;}
	virtual bool     getStk() const {return stk;}
	virtual void     setPlm(bool   plm_){plm = plm_;}
	virtual bool     getPlm() const {return plm;}
	virtual void     setEmi(double emi_){emi = emi_;}
	virtual double   getEmi() const {return emi;}
	virtual void     setVel(double vel_){vel = vel_;}
	virtual double   getVel() const {return vel;}
	virtual void     setGas(double gas_){gas = gas_;}
	virtual double   getGas() const {return gas;}
	virtual void     setTpr(double tpr_){tpr = tpr_;}
	virtual double   getTpr() const {return tpr;}
	virtual void     setDia(double dia_){dia = dia_;}
	virtual double   getDia() const {return dia;}
	virtual void     setCtr(Vector3d ctr_){ctr = ctr_;}
	virtual Vector3d getCtr() const {return ctr;}

	virtual void     setIde(int ide_){ide = ide_;}
	virtual int      getIde() const {return ide;}
	virtual void     setAlt(double alt_){alt = alt_;}
	virtual double   getAlt() const {return alt;}

	virtual double dh() const;
	virtual double stacktip(double ws_modified_with_power_law) const;
	virtual double concawe (double ws_modified_with_power_law) const;
	virtual double briggs  () const;

	virtual void   setDhAtStackPosition(double dH_){dH = dH_;}
	virtual double getDhAtStackPosition() const   {return dH;}
	virtual void   evaluateDhAtStackPosition(double ws_modified_with_power_law);

	virtual double wsMltfactor() const;
public:
	PointSrcSimulator(){setOpenAngle(100.0);}
	virtual ~PointSrcSimulator(){}
	virtual void scan ();
	virtual void print();
	virtual void lists();
	virtual void exec (bool new_met, bool new_src);
};


#endif