#include "SimulationFactory.h"
#include "ErrorHandler.h"
#include "showcase.h"

void SimulationFactory::scan()
{
	ML_TRACE("SimulationFactory::scan");

	string explanation;

	try{
		if(false){/* dummy */}
		else if(cls == "ln")
		{
			explanation = "Line Source Simulator";
			simulator   = new LineSrcNumItgSimulatorAnotherIntegrand();
		}
		else if(cls == "mt")
		{
			explanation = "Enhanced MetiLis Point Source Simulator";
			simulator   = new MetiLisPointDropletSrcSimulator();
		}
		/*
		*
		*	Add here your own custom-made simulator classes.
		*
		*/
		else throw 1;
	}
	catch(int checkpoint)
	{
		switch(checkpoint)
		{
		case 1:
			cerr << ErrorSimulationFactory_scan(explanation);
			CERR << ErrorSimulationFactory_scan(explanation);
			break;
		}
		exit(1);
	}

	clog << "+SIMUTATOR\n"; clog << " - Running " << explanation << " ...\n\n";
	CLOG << "+SIMULATOR\n"; CLOG << " - Running " << explanation << " ...\n\n";

	simulator->setTimeSeriesFlag(is_time_series);
	simulator->scan();

	level = simulator->getListLevel();
}

void SimulationFactory::print()
{
	ML_TRACE("SimulationFactory::print");

	simulator->print();
}

void SimulationFactory::lists()
{
	ML_TRACE("SimulationFactory::lists");

	simulator->lists();
}

void SimulationFactory::loop()
{
	ML_TRACE("SimulationFactory::loop");




	Met* ptr_met = simulator->getMetPtr();
	Rcp* ptr_rcp = simulator->getRcpPtr();
	Src* ptr_src = simulator->getSrcPtr();
	Blg* ptr_blg = simulator->getBlgPtr();


	if(cls == "mt" && ptr_blg->getNoElms() != ptr_src->getNoElms())
	{
		CERR << "SimulationFactory::loop" << endl;
		CERR << "The number of data is Mismatched." << endl;
		CERR << "Check the first line of buildingparameters.in.csv and pointsource.in.csv." << endl;
		exit(1);
	}


	Pat* ptr_pat = simulator->getPatPtr();
	ptr_pat->open();

	if(ptr_pat->getNoElms() != ptr_src->getNoElms())
	{
		CERR << "SimulationFactory::loop" << endl;
		CERR << "The number of data is Mismatched." << endl;
		CERR << "Check the first line of pattern.in.csv and pointsource.in.csv." << endl;
		exit(1);
	}

	Ran* ptr_ran = simulator->getRanPtr();
	ptr_ran->open();



	const int num_of_steps = (ptr_met->getNoElms())*(ptr_src->getNoElms());
	char buffer[50]; _itoa(num_of_steps, buffer, 10);
	const string str_num_of_steps = buffer;
	clog << "+CHECKING TOTAL STEPS ... "; clog << str_num_of_steps + " steps counted.\n\n";
	CLOG << "+CHECKING TOTAL STEPS ... "; CLOG << str_num_of_steps + " steps counted.\n\n";


	const double progress = simulator->getProgressPeekStep();
	int counter = 0;
	int steps = 0;
	ptr_met->head();
	for(int i = 1; i <= ptr_met->getNoElms(); i++)
	{
		ptr_pat->scan();
		ptr_ran->scan();

		if((level == Simulator::MODERATE || level == Simulator::FULL))
		{
			is_time_series ? ptr_pat->lists(cls, ptr_met) : ptr_pat->lists(cls, i);
		}

		bool new_met = true;
		if(!(simulator->isBadData()))
		{

			ptr_rcp->init();
			ptr_src->head();
			ptr_pat->head();
			if(cls == "mt") ptr_blg->head();
			for(int j = 1; j <= ptr_src->getNoElms(); j++)
			{
				double percent;
				ptr_pat->getPat(percent);
				ptr_src->setOperationPercent(percent);
				bool new_src = true;

				ptr_rcp->head();
				for(int k = 1; k <= ptr_rcp->getNoElms(); k++)
				{

					ptr_ran->head();
					for(int l = 1; l <= ptr_ran->getNoElms(); l++)
					{
						simulator->exec(new_met, new_src);
						if(new_met) new_met = false;
						if(new_src) new_src = false;
						simulator->addSrcContrib(1.0/ptr_ran->getNoElms());
						ptr_ran->advance();
					}
					ptr_rcp->advance();
				}
				ptr_src->advance();
				ptr_pat->advance();
				if(cls == "mt") ptr_blg->advance();

				steps++;
				clog << ErrorSimulationFactory_loop(num_of_steps, progress,  steps, false);
				CLOG << ErrorSimulationFactory_loop(num_of_steps, progress,  steps,  true);
			}
			simulator->assembleResult(++counter);
		}
		else
		{
			if((level == Simulator::MODERATE || level == Simulator::FULL))
			{
				CLST << " ******* Warning! Skipping calculation at this step because of the defective meteorological data.";
			}

			steps += ptr_src->getNoElms();
		}

		ptr_met->advance();
	}
	ptr_ran->close();
	ptr_pat->close();
}

void SimulationFactory::dump()
{
	ML_TRACE("SimulationFactory::dump");

	Rcp* ptr_rcp = simulator->getRcpPtr();

	if(simulator->isTimeSeries())
	{
		const int nr_dy_codes_p = simulator->getMaxDailyCodes() + 1;
		for(int dy_code = 1; dy_code <= nr_dy_codes_p; dy_code++)
		{
			const int nr_hr_codes_p = simulator->getMaxHourlyCodes() + 1;
			for(int hr_code = 1; hr_code <= nr_hr_codes_p; hr_code++)
			{
				const int nr_elms = ptr_rcp->getNoElms();
				ptr_rcp->head();
				for(int i = 1; i <= nr_elms; i++)
				{
					const int n1 = (i-1) + (hr_code-1)*nr_elms + (dy_code-1)*nr_elms*nr_hr_codes_p;
					const int n2 = (hr_code-1) + (dy_code-1)*nr_hr_codes_p;
					ptr_rcp->valuePt(Rcp::CON) = (simulator->lt_res[n1])/(simulator->lt_nrs[n2]);
					ptr_rcp->valuePt(Rcp::DEP) = (simulator->lt_dep[n1])/(simulator->lt_nrs[n2]);
					ptr_rcp->valuePt(Rcp::FLG) = (simulator->lt_flg[n1] != 0.0 ? 1.0 : 0.0);
					ptr_rcp->advance();
				}
				ptr_rcp->head();

				const string old = ptr_rcp->getOsFile(); char buf[2]; string filename;
				_itoa(dy_code, buf, 10); filename = buf;
				_itoa(hr_code, buf, 10); filename = filename + "x" + buf;
				filename = "../tmp/" + simulator->getPrefix() + "_" + filename + ".out.csv";
				ptr_rcp->setOsFile(filename);
				ptr_rcp->print(true); ptr_rcp->lists(dy_code, hr_code, (level == Simulator::MODERATE || level == Simulator::FULL));
				ptr_rcp->setOsFile(old);
			}
		}
	}
}
