#include "Blg.h"
#include "ErrorHandler.h"

int _Blg_::get_Blg_(double& w1_, double& w2_, double& hh_, int& tp_, double& he_, double& dx_, double& dy_, double& z1_, double& z2_, double& y1_, double& y2_, double& al_, int& ef_) const
{
	ML_TRACE("_Blg_::get_Blg_");

	w1_ = w1[pwa];
	w2_ = w2[pwa];
	hh_ = hh[pwa];
	tp_ = tp[pwa];
	he_ = he[pwa];
	dx_ = dx[pwa];
	dy_ = dy[pwa];
	z1_ = z1[pwa];
	z2_ = z2[pwa];
	y1_ = y1[pwa];
	y2_ = y2[pwa];
	al_ = al[pwa];
	ef_ = ef[pwa];

	return sq[pwa];
}

void _Blg_:: scan(ifstream& is)
{
	ML_TRACE("_Blg_:: scan");

	for(int i = 0; i < 16; i++)
	{
		is >> sq[i];
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is >> wd[i];
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is >> w1[i];
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is >> w2[i];
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is >> hh[i];
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is >> tp[i];
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is >> he[i];
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is >> dx[i];
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is >> dy[i];
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is >> z1[i];
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is >> z2[i];
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is >> y1[i];
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is >> y2[i];
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is >> al[i];
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is >> ef[i];
	}
}

void _Blg_::print(ofstream& os)
{
	ML_TRACE("_Blg_::print");

	for(int i = 0; i < 16; i++)
	{
		os << setw(COLUMN);
		os << sq[i] << ",";
		os << setw(COLUMN);
		os << wd[i] << ",";
		os << setw(COLUMN);
		os << w1[i] << ",";
		os << setw(COLUMN);
		os << w2[i] << ",";
		os << setw(COLUMN);
		os << hh[i] << ",";
		os << setw(COLUMN);
		os << tp[i] << ",";
		os << setw(COLUMN);
		os << he[i] << ",";
		os << setw(COLUMN);
		os << dx[i] << ",";
		os << setw(COLUMN);
		os << dy[i] << ",";
		os << setw(COLUMN);
		os << z1[i] << ",";
		os << setw(COLUMN);
		os << z2[i] << ",";
		os << setw(COLUMN);
		os << y1[i] << ",";
		os << setw(COLUMN);
		os << y2[i] << ",";
		os << setw(COLUMN);
		os << al[i] << ",";
		os << setw(COLUMN);
		os << ef[i] << endl;
	}
}

void _Blg_::lists()
{
	ML_TRACE("_Blg_::lists");

	for(int i = 0; i < 16; i++)
	{
		CLST << setw(COLUMN);
		CLST << sq[i];

		double wdm = 22.5*(i+1) - 11.25; wdm = (wdm < 0   ? wdm + 360 : wdm);
		double wdp = 22.5*(i+1) + 11.25; wdp = (wdp > 360 ? wdp - 360 : wdp);
		CLST << setw(COLUMN);
		CLST << setw(3) << wd[i] << " (" << setw(6) << wdm << "->" << setw(6) << wdp << ')';

		CLST << setw(COLUMN);
		CLST << w1[i];
		CLST << setw(COLUMN);
		CLST << w2[i];
		CLST << setw(COLUMN);
		CLST << hh[i];
		CLST << setw(COLUMN);
		CLST << tp[i];
		CLST << setw(COLUMN);
		CLST << he[i];
		CLST << setw(COLUMN);
		CLST << dx[i];
		CLST << setw(COLUMN);
		CLST << dy[i];
		CLST << setw(COLUMN);
		CLST << z1[i];
		CLST << setw(COLUMN);
		CLST << z2[i];
		CLST << setw(COLUMN);
		CLST << y1[i];
		CLST << setw(COLUMN);
		CLST << y2[i];
		CLST << setw(COLUMN);
		CLST << al[i];
		CLST << setw(COLUMN);
		CLST << ef[i] << endl;
	}
}

Blg::~Blg()
{
	ML_TRACE("Blg::~Blg");

	if(getNoElms() > 0)
	{
		head();
		delete [] blg;
		setNoElms(0); setPosition(0);
	}
}

void Blg::redim(int n)
{
	ML_TRACE("Blg::redim");

	if(getNoElms() != 0)
	{
		head();
		delete [] blg;
		setNoElms(0); setPosition(0);
	}
	blg = new _Blg_ [n+1];
	setNoElms(n); setPosition(1);
}

void Blg::advance()
{
	SimpleIterator::advance();
	blg++;
}

void Blg::reverse()
{
	SimpleIterator::reverse();
	blg--;
}

void Blg::scan()
{
	ML_TRACE("Blg::scan");

	const string filename = getIsFile();
	ifstream  is(filename.c_str());

	is.exceptions(ios_base::goodbit);
	try{
		is.exceptions(ios_base::badbit | ios_base::failbit);

		int n; is >> n;

		redim(n);
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');

		head();
		for(int i = 1; i <= getNoElms(); i++)
		{
			blg->scan(is);
			advance();
		}
		head();
		is.close();
	}
	catch(...)
	{
		cerr << ErrorBlg_scan(filename);
		CERR << ErrorBlg_scan(filename);
		exit(1);
	}
}

void Blg::print()
{
#if SCAN_CHECK > 0
	const string filename = getOsFile();
	ofstream  os(filename.c_str());

	head();
	for(int i = 1; i <= getNoElms(); i++)
	{
		blg->print(os);
		advance();
	}
	head();
	os.close();
#endif
}

void Blg::lists()
{
	ML_TRACE("Blg::lists");

	CLST << endl;
	CLST << endl;
	CLST << "<< Evaluated building parameters list >>" << endl;
	CLST << endl;
	CLST << "+ DIFFUSION PARAMETERS RELATED TO BUILDING DOWNWASH +" << endl;
	CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
	CLST << setw(COLUMN) << "Stack IDs" << setw(COLUMN) << "Wind dir (1-16)" << setw(COLUMN) << "Bldg Width 1 (m)" << setw(COLUMN) << "Bldg Width 2 (m)" << setw(COLUMN) << "Bldg Height (m)" << setw(COLUMN) << "Cluster type (1-2)" << setw(COLUMN) << "Wake He (m)" << setw(COLUMN) << "Stack transl. x (m)" << setw(COLUMN) << "Stack transl. y (m)" << setw(COLUMN) << "Vert param (Cz1)" << setw(COLUMN) << "Vert param (Cz2)" << setw(COLUMN) << "Horiz param (Cy1)" << setw(COLUMN) << "Horiz param (Cy2)" << setw(COLUMN) << "Wake wind factor" << setw(COLUMN) << "Bldg IDs" << endl;
	CLST << "------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------" << endl;
	head();
	for(int i = 1; i <= getNoElms(); i++)
	{
		blg->lists();
		CLST << "------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------" << endl;
		advance();
	}
	head(); CLST << endl; CLST << endl;
}

int Blg::getBlg(double& w1_, double& w2_, double& hh_, int& tp_, double& he_, double& dx_, double& dy_, double& z1_, double& z2_, double& y1_, double& y2_, double& al_, int& ef_) const
{
	ML_TRACE("Blg::get_Blg");
	return blg->get_Blg_(w1_, w2_, hh_, tp_, he_, dx_, dy_, z1_, z2_, y1_, y2_, al_, ef_);
}
