#include "droplet.h"
#include "ErrorHandler.h"
#include "../share/common.h"

Droplet::~Droplet()
{
	ML_TRACE("Droplet::~Droplet");

	if(getNoElms() > 0)
	{
		head();
		delete [] droplet_inf;
		delete [] droplet_sup;
		setNoElms(0); setPosition(0);
	}
}

void Droplet::redim(int n)
{
	ML_TRACE("Droplet::redim");

	if(getNoElms() > 0)
	{
		head();
		delete [] droplet_inf;
		delete [] droplet_sup;
		setNoElms(0); setPosition(0);
	}

	droplet_inf = new double[n+1];
	droplet_sup = new double[n+1];
	setNoElms(n); setPosition(1);
}

void Droplet::advance()
{
	SimpleIterator::advance();
	droplet_inf++;
	droplet_sup++;
}

void Droplet::reverse()
{
	SimpleIterator::reverse();
	droplet_inf--;
	droplet_sup--;
}

void Droplet::scan()
{
	ML_TRACE("Droplet::scan");

	const string filename = getIsFile();
	ifstream  is(filename.c_str());

	is.exceptions(ios_base::goodbit);
	try{
		is.exceptions(ios_base::badbit | ios_base::failbit);

		int n; is >> n; redim(n);
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');

		head();
		while(getPosition() <= getNoElms())
		{
			double _droplet_inf;
			double _droplet_sup;

			is >> _droplet_inf;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _droplet_sup;

			*droplet_inf = _droplet_inf;
			*droplet_sup = _droplet_sup;
			advance();
		}
		head();
		is.close();
	}
	catch(...)
	{
		cerr << ErrorDroplet_scan(filename);
		CERR << ErrorDroplet_scan(filename);
		exit(1);
	}
}

void Droplet::print()
{
#if SCAN_CHECK > 0
	const string filename = getOsFile();
	ofstream  os(filename.c_str());

	os << "<< Scanned meteorological data (inf|sup). " << getNoElms() << " data readed. >>" << "," << endl;
	os << endl;

	head();
	while(getPosition() <= getNoElms())
	{
		os << setw(COLUMN) << *droplet_inf << ',' << setw(COLUMN) << *droplet_sup << endl;
		advance();
	}
	head();
	os.close();
#endif
}

void Droplet::lists()
{
	ML_TRACE("Droplet::lists");

	CLST << endl;
	CLST << endl;
	CLST << "<< Droplet diameter class definition >>" << endl;
	CLST << endl;
	CLST << "+ DROPLET DIAMETER CLASSES (Droplet having diameter less than 10 microns regarded as gaseous materials) +"    << endl;
	CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^"    << endl;
	CLST << setw(COLUMN) << "Lower (microns)" << setw(COLUMN) << "Upper (microns)" << setw(COLUMN) << "Average (microns)" << endl;
	CLST << "------------------------------------------------------------" << endl;
	head();
	while(getPosition() <= getNoElms())
	{
		CLST << setw(COLUMN) << (*droplet_inf)*pow(10, 6) << setw(COLUMN) << (*droplet_sup)*pow(10, 6) << setw(COLUMN) << 0.5*(*droplet_inf + *droplet_sup)*pow(10, 6) << endl;
		advance();
	}
	head(); CLST << endl;
}
