#include "pat.h"
#include "ErrorHandler.h"
#include "../share/common.h"

Pat::~Pat()
{
	ML_TRACE("Pat::~Pat");

	if(getNoElms() > 0)
	{
		head();
		delete [] pt;
		setNoElms(0); setPosition(0);
	}
}

void Pat::open()
{
	ML_TRACE("Pat::open");

	const string filename = getIsFile(); is.open(filename.c_str());

	is.exceptions(ios_base::goodbit);
	try{
		is.exceptions(ios_base::badbit | ios_base::failbit);

		int n; is >> n; redim(n);
		for(int i = 1; i < n; i++)
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		head();
	}
	catch(...)
	{
		cerr << ErrorPat_open(filename);
		CERR << ErrorPat_open(filename);
		exit(1);
	}
}

void Pat::close()
{
	ML_TRACE("Pat::close");

	is.close();
}

void Pat::setPat(double  pt_)
{
	ML_TRACE("Pat::setPat");

	*pt = pt_;
}

void Pat::getPat(double& pt_) const
{
	ML_TRACE("Pat::getPat");

	pt_ = *pt;
}

void Pat::redim(int n)
{
	ML_TRACE("Pat::redim");

	if(getNoElms() > 0)
	{
		head();
		delete [] pt;
		setNoElms(0); setPosition(0);
	}

	pt = new double[n+1];
	setNoElms(n); setPosition(1);
}

void Pat::advance()
{
	SimpleIterator::advance();
	pt++;
}

void Pat::reverse()
{
	SimpleIterator::reverse();
	pt--;
}

void::Pat::scan()
{
	ML_TRACE("Pat::scan");

	is.exceptions(ios_base::goodbit);
	try{
		is.exceptions(ios_base::badbit | ios_base::failbit);

		head();
		while(getPosition() <= getNoElms())
		{
			double _pt;
			is >>  _pt; if(getPosition() != getNoElms()) is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			*pt =  _pt; advance();
		}
		head();
	}
	catch(...)
	{
		const string filename = getIsFile();
		cerr << ErrorPat_scan(filename);
		CERR << ErrorPat_scan(filename);
		exit(1);
	}
}

void Pat::print()
{
#if SCAN_CHECK > 0
	const string filename = getOsFile();
	ofstream  os(filename.c_str());

	head();
	while(getPosition() <= getNoElms())
	{
		os << setw(COLUMN) << *pt << ',';
		advance();
	}
	head();
	os << endl; os.close();
#endif
}

void Pat::lists(string cls, int  cas)
{
	ML_TRACE("Pat::lists");

	CLST << endl;
	CLST << "+ Operation Rates in CASE" << setw(5) << cas;
	CLST << endl;
	CLST << "\tScanned : [" << setw(3) << "IDs" << ": " << setw(3) << "(%)" << "]";
	if(cls == "mt") CLST << "       ";

	head();
	for(int i = 1; i <= getNoElms(); i++)
	{
		CLST << "[" << setw(3) << i << ":" << setw(3) << floor(*pt + 0.5) << "]";
		if(cls == "mt") CLST << "       ";
		advance();
	}
	head(); CLST << endl;

	if(cls == "mt")
	{
		CLST << "\tApplied : <   dh(m)>";
	}
	else if(cls == "ln")
	{
		CLST << "\tApplied :           ";
	}
}

void Pat::lists(string cls, Met* met)
{
	ML_TRACE("Pat::lists");

	// Placeholders for retrieving the current time
	int _yr, _mn, _dy, _hr; met->getTime(_yr, _mn, _dy, _hr);

	CLST << endl;
	CLST << "+ Operation Rates at " << setw(4) << _yr << "/" << setw(2) << _mn << "/" << setw(2) << _dy << " : " << setw(2) << _hr << ":" << setw(2) << "00";
	CLST << endl;
	CLST << "\tScanned : [" << setw(3) << "IDs" << ": " << setw(3) << "(%)" << "]";
	if(cls == "mt") CLST << "       ";

	head();
	for(int i = 1; i <= getNoElms(); i++)
	{
		CLST << "[" << setw(3) << i << ":" << setw(3) << floor(*pt + 0.5) << "]";
		if(cls == "mt") CLST << "       ";
		advance();
	}
	head(); CLST << endl;

	if(cls == "mt")
	{
		CLST << "\tApplied : <   dh(m)>";
	}
	else if(cls == "ln")
	{
		CLST << "\tApplied :           ";
	}
}
