#ifndef RCP_H
#define RCP_H

#include "../share/SimpleIterator.h"
#include "../share/WildMagic.h"
#include "../share/FileFactory.h"

class Rcp : public SimpleIterator, public FileManager
{
  Vector3d* vec;
  string*   x__;
  string*   y__;
  string*   z__;
  double*   hhh;  // Altitude
  string*   cmm;  // Description
  double*   va1;  // Concentration value
  double*   va2;  // Deposition value
  double*   flg;  // Validation flag (1: Validated, 0: NOT Validated)
  int       opt;  // Altitude data consideration flag (1: Applied, 0: Ignored)

  bool      concentration_unit_flag;
 public:
  Rcp()
    {
      setIsFile(FileList::fnRcp(FileList::IS, 1));
      setOsFile(FileList::fnRcp(FileList::OS, 1));
    }
  virtual ~Rcp();
  enum Result {CON, DEP, FLG};
  double  valuePt(Result res) const {return (res == CON ? *va1 : (res == DEP ? *va2 : *flg));}
  double& valuePt(Result res)       {return (res == CON ? *va1 : (res == DEP ? *va2 : *flg));}
  void setCoor(double  x, double  y, double  z, double  h);
  void getCoor(double& x, double& y, double& z, double& h) const;
  void setTerrainOpt(int opt_) {opt = opt_;}
  int  getTerrainOpt() const   {return opt;}
  virtual void redim(int n);
  virtual void init ();
  virtual void scan ();
  virtual void print(bool b_val = false);
  virtual void lists();
  virtual void lists(int cs_code, bool verbose);
  virtual void lists(int dy_code, int hr_code, bool verbose);
  virtual void advance();
  virtual void reverse();

  virtual void   setConcentrationUnitFlag(bool concentration_unit_flag_){concentration_unit_flag = concentration_unit_flag_;}
  virtual bool   getConcentrationUnitFlag() const {return concentration_unit_flag;}
};

#endif
