#include "trapezoid.h"

void Trapezoid::trapezoid(int n)
{
	ML_TRACE("Trapezoid::trapezoid");

	const double inf = data->getInf();
	const double sup = data->getSup();

	double x,tnm,sum,del;
	int it,j;

	if (n == 1)
		cur = 0.5*(sup - inf)*(integrand(inf) + integrand(sup));
	else
	{
		for (it = 1, j = 1; j < n-1; j++)
		{
			it <<= 1;
		}

		tnm = it;
		del = (sup - inf)/tnm;
		x = inf + 0.5*del;

		for (sum = 0.0, j = 1; j <= it; j++, x += del)
		{
			sum += integrand(x);
		}

		cur = 0.5*(cur + (sup - inf)*sum/tnm);
	}
}

double Trapezoid::run()
{
	ML_TRACE("Trapezoid::run");

 	prv = -1.0e30;
 	for(int i = 1; i <= data->getMaximumIterations(); i++)
	{
		trapezoid(i);
		if(fabs(cur - prv) < (data->getToleranceRatio())*fabs(prv))
		{
			return cur;
		}
		prv = cur;
	}

	exit(1);
}
